import pyrkube
from pyrkube.hosts import get_hostname, PodHostname, StatefulPodHostname

from . import TestCase


class TestPodHostsFromString(TestCase):
    def setUp(self):
        self.ip = '172.16.16.223'
        self.hostname = self.ip.replace('.', '-')
        self.namespace = 'default'
        self.domain = 'cluster.local'
        self.hostname_str = '%s.%s.pod.%s' % (
            self.hostname, self.namespace, self.domain
        )

    def tearDown(self):
        self.ip = None
        self.hostname = None
        self.namespace = None
        self.domain = None
        self.hostname_str = None

    def test_create(self):
        h = get_hostname(self.hostname_str)
        self.assertTrue(repr(h))
        self.assertEqual(type(h), PodHostname)

    def test_properties(self):
        h = get_hostname(self.hostname_str)

        self.assertEqual(h.ip, self.ip)
        self.assertEqual(h.hostname, self.hostname)
        self.assertEqual(h.namespace, self.namespace)
        self.assertEqual(h.domain, self.domain)
        self.assertEqual(h.fqdn, self.hostname_str)

    def test_string_conversion(self):
        h = get_hostname(self.hostname_str)

        self.assertEqual(type(str(h)), str)
        self.assertEqual(str(h), self.hostname_str)

    def test_clone(self):
        h1 = get_hostname(self.hostname_str)
        h2 = h1.clone()
        self.assertEqual(repr(h1), repr(h2))
        self.assertEqual(str(h1), str(h2))

    def test_sort(self):
        hosts = [get_hostname('172-17-0-%d.default.pod.cluster.local' % i)
                 for i in range(3, 0, -1)]

        self.assertFalse(
            str(hosts[0]) < str(hosts[1]) < str(hosts[2])
        )

        hosts = sorted(hosts)
        self.assertTrue(
            str(hosts[0]) < str(hosts[1]) < str(hosts[2])
        )


class TestPodHostsFromPod(TestCase):
    def setUp(self):
        self.env = pyrkube.config.ENVIRONMENT
        self.api = pyrkube.client.KubeAPIClient(
            env=self.env, namespace='default', domain='cluster.local')
        self.ep = self.api.get('endpoint', 'rabbitmq')
        self.pods = self.ep.pods

    def tearDown(self):
        self.env = None
        self.api = None
        self.ep = None
        self.pods = None

    def test_create(self):
        for pod in self.pods:
            h = pyrkube.hosts.get_hostname(pod)
            self.assertTrue(repr(h))
            self.assertEqual(type(h), PodHostname)

    def test_properties(self):
        for pod in self.pods:
            h = pyrkube.hosts.get_hostname(pod)
            ip = pod.status.podIP
            hostname = ip.replace('.', '-')
            namespace = pod.metadata.namespace
            domain = 'cluster.local'
            fqdn = '%s.%s.pod.%s' % (hostname, namespace, domain)

            self.assertEqual(h.ip, ip)
            self.assertEqual(h.hostname, hostname)
            self.assertEqual(h.namespace, namespace)
            self.assertEqual(h.domain, domain)
            self.assertEqual(h.fqdn, fqdn)

    def test_string_conversion(self):
        for pod in self.pods:
            h = pyrkube.hosts.get_hostname(pod)
            ip = pod.status.podIP
            hostname = ip.replace('.', '-')
            namespace = pod.metadata.namespace
            domain = 'cluster.local'
            fqdn = '%s.%s.pod.%s' % (hostname, namespace, domain)
            self.assertTrue(repr(h))

            self.assertEqual(type(str(h)), str)
            self.assertEqual(str(h), fqdn)

    def test_clone(self):
        for pod in self.pods:
            h1 = pyrkube.hosts.get_hostname(pod)

            h2 = h1.clone()
            self.assertEqual(repr(h1), repr(h2))
            self.assertEqual(str(h1), str(h2))


class TestStatefulHostsFromString(TestCase):
    def setUp(self):
        self.index = 0
        self.statefulset = 'couchdb'
        self.hostname = '%s-%d' % (self.statefulset, self.index)
        self.subdomain = 'couchdb'
        self.namespace = 'default'
        self.domain = 'cluster.local'
        self.hostname_str = '%s.%s.%s.svc.%s' % (
            self.hostname, self.subdomain, self.namespace, self.domain
        )
        self.host = get_hostname(self.hostname_str)

    def tearDown(self):
        self.index = None
        self.statefulset = None
        self.hostname = None
        self.subdomain = None
        self.namespace = None
        self.domain = None
        self.hostname_str = None
        self.host = None

    def test_create(self):
        h = get_hostname(self.hostname_str)
        self.assertTrue(repr(h))
        self.assertEqual(type(h), StatefulPodHostname)

    def test_properties(self):
        h = self.host
        self.assertEqual(h.index, self.index)
        self.assertEqual(h.statefulset, self.statefulset)
        self.assertEqual(h.hostname, self.hostname)
        self.assertEqual(h.subdomain, self.subdomain)
        self.assertEqual(h.namespace, self.namespace)
        self.assertEqual(h.domain, self.domain)
        self.assertEqual(h.fqdn, self.hostname_str)

    def test_string_conversion(self):
        h = self.host

        self.assertEqual(type(str(h)), str)
        self.assertEqual(str(h), self.hostname_str)

    def test_clone(self):
        h1 = self.host
        h2 = h1.clone()
        self.assertEqual(repr(h1), repr(h2))
        self.assertEqual(str(h1), str(h2))

        h3 = h1.clone(index=1)
        self.assertEqual(h3.index, 1)
        h4 = h3.clone(master=True)
        self.assertEqual(h4.index, 0)

    def test_sort(self):
        hosts = [self.host.clone(index=i) for i in range(3, 0, -1)]
        self.assertFalse(
            str(hosts[0]) < str(hosts[1]) < str(hosts[2])
        )
        hosts = sorted(hosts)
        self.assertTrue(
            str(hosts[0]) < str(hosts[1]) < str(hosts[2])
        )

    def test_is_master(self):
        h = self.host
        self.assertTrue(h.is_master)
        h2 = h.clone(index=1)
        self.assertFalse(h2.is_master)


class TestStatefulHostFromPod(TestCase):
    def setUp(self):
        self.env = pyrkube.config.ENVIRONMENT
        self.api = pyrkube.client.KubeAPIClient(
            env=self.env, namespace='default', domain='cluster.local')
        self.ep = self.api.get('endpoint', 'couchdb')
        self.pods = self.ep.pods

        self.statefulset = 'couchdb'
        self.subdomain = 'couchdb'
        self.namespace = 'default'
        self.domain = 'cluster.local'

    def tearDown(self):
        self.env = None
        self.api = None
        self.ep = None
        self.pods = None

        self.statefulset = None
        self.subdomain = None
        self.namespace = None
        self.domain = None

    def test_create(self):
        for pod in self.pods:
            h = get_hostname(pod)
            self.assertTrue(repr(h))
            self.assertEqual(type(h), StatefulPodHostname)

    def test_properties(self):
        for pod in self.pods:
            h = get_hostname(pod)
            ip = pod.status.podIP
            index = int(pod.metadata.name.split('-')[1])
            hostname = '%s-%d' % (self.statefulset, index)

            fqdn = '%s-%d.%s.%s.svc.%s' % (
                self.statefulset,
                index,
                self.subdomain,
                self.namespace,
                self.domain
            )
            self.assertEqual(h.ip, ip)
            self.assertEqual(h.index, index)
            self.assertEqual(h.statefulset, self.statefulset)
            self.assertEqual(h.hostname, hostname)
            self.assertEqual(h.subdomain, self.subdomain)
            self.assertEqual(h.namespace, self.namespace)
            self.assertEqual(h.domain, self.domain)
            self.assertEqual(h.fqdn, fqdn)

    def test_string_conversion(self):
        for pod in self.pods:
            h = get_hostname(pod)
            index = int(pod.metadata.name.split('-')[1])

            fqdn = '%s-%d.%s.%s.svc.%s' % (
                self.statefulset,
                index,
                self.subdomain,
                self.namespace,
                self.domain
            )

            self.assertEqual(type(str(h)), str)
            self.assertEqual(str(h), fqdn)

    def test_clone(self):
        for pod in self.pods:
            h1 = get_hostname(pod)

            h2 = h1.clone()
            self.assertEqual(repr(h1), repr(h2))
            self.assertEqual(str(h1), str(h2))

            h3 = h1.clone(index=1)
            self.assertEqual(h3.index, 1)
            h4 = h3.clone(master=True)
            self.assertEqual(h4.index, 0)

    def test_sort(self):
        hosts = self.ep.hostnames
        self.assertTrue(
            str(hosts[0]) < str(hosts[1]) < str(hosts[2])
        )

    def test_is_master(self):
        for pod in self.pods:
            h = get_hostname(pod)
            index = int(pod.metadata.name.split('-')[1])

            if index == 0:
                self.assertTrue(h.is_master)
            else:
                self.assertFalse(h.is_master)



"""
pyrkube.client
~~~~~~~~~~~~~~

API Client wrappers, etc for pyrkube.

:copyright: (c) 2016 by Joe Black.
:license: Apache2.
"""

import collections

import pykube

from . import config, objects
from .adict import adict


SERVICE_ACCOUNT_PATH = '/var/run/secrets/kubernetes.io/serviceaccount'

WatchEvent = collections.namedtuple('WatchEvent', 'type object')


class KubeAPIClient:
    """A general-case client for interacting with kubernetes.

    Wraps the pykube library in a manner that is more similar to frequently
    used kubectl commands.
    """

    def __init__(self, env='dev', namespace='default', domain='cluster.local'):
        self.env = env
        self.namespace = namespace
        self.domain = domain
        self.api = self._get_api()

    @property
    def context(self):
        """Return the current kube context."""
        return self.api.config.contexts[self.api.config.current_context]

    @property
    def cluster(self):
        """Return the current kube cluster."""
        return self.context['cluster']

    @property
    def user(self):
        """Return the current kube user."""
        return self.context['user']

    @property
    def server(self):
        """Return the current kube server."""
        return self.api.config.cluster['server']

    def __repr__(self):
        return '%s(cluster: %s, user: %s, server: %s)' % (
            type(self).__name__, self.cluster, self.user, self.server
        )

    def _get_api(self):
        try:
            return pykube.http.HTTPClient(
                pykube.KubeConfig.from_service_account())
        except FileNotFoundError:
            return pykube.http.HTTPClient(
                pykube.KubeConfig.from_file('~/.kube/config'))

    @staticmethod
    def _get_resource_name(name):
        keys = list(objects.__dict__.keys())
        for key in keys:
            if key.lower() == name.lower():
                return key

    def _get_resources(self, name):
        name = self._get_resource_name(name)
        return adict(
            pykube=getattr(pykube.objects, name),
            pyrkube=getattr(objects, name)
        )

    def get(self, resource, name=None, selector=None, namespace=None,
            watch=False, since=None):
        """Return an api object.

        Attributes:
          resource: (str) the pyrkube resource type to query
          name: (str) the metadata.name of the object to return
          selector: (dict) the label selector to query with
          namespace: (str) the namespace from which to query
          watch: (bool) return a watch query instead of just resource(s)
          since: (string) a resource version to pass to the watch query

        Returns:
          Either a Resource Object, list of Resource Objects, or if a watch
          query, a generator of Watch EVents.
        """
        namespace = namespace or self.namespace
        resources = self._get_resources(resource)
        if not issubclass(resources.pykube, pykube.objects.APIObject):
            raise pykube.PyKubeError(
                'No pykube object of type: %s', resources.pykube
            )

        req = pykube.query.Query(
            self.api, resources.pykube, namespace=namespace
        )
        if selector:
            req = req.filter(selector=selector)
            if watch:
                req = req.watch(since)
        if name:
            req = req.get_or_none(name=name)
        if req:
            return self._wrap_resource(req, resources.pyrkube, namespace)

    def _wrap_resource(self, request, resource, namespace):
        api = self.clone(namespace)
        if isinstance(request, pykube.query.WatchQuery):
            return self._return_watch(api, request, resource)
        elif isinstance(request, pykube.query.Query):
            return [resource(api, obj) for obj in request.all()]
        elif isinstance(request, pykube.objects.APIObject):
            return resource(api, request)

    @staticmethod
    def _return_watch(api, request, resource):
        for evt in request:
            yield WatchEvent(evt.type, resource(api, evt.object))

    def clone(self, namespace=None):
        """Clone the api, optionally with a different namespace."""
        namespace = namespace or self.namespace
        return type(self)(self.env, namespace)


class KubeInterfaceBase:
    """Base Template for Kube Interface."""

    def __init__(self, environment=None, namespace=None, domain=None):
        self.environment = environment or config.ENVIRONMENT
        self.domain = domain or config.DOMAIN
        self.namespace = namespace or self._get_namespace()
        self.api = KubeAPIClient(self.environment, self.namespace)

    @staticmethod
    def _get_namespace():
        try:
            with open(SERVICE_ACCOUNT_PATH + '/namespace') as fd:
                return fd.read().strip()
        except FileNotFoundError:
            return 'default'



"""
pyrkube.objects
~~~~~~~~~~~~~~

API Resource objects for pyrkube.

:copyright: (c) 2016 by Joe Black.
:license: Apache2.
"""

import copy
import base64
import logging
import time
import abc

from . import util, hosts
from .adict import adict


class GenericBase(adict, metaclass=abc.ABCMeta):
    """Generic object base class."""
    def __init__(self, obj, *args, **kwargs):
        adict.__init__(self, obj, *args, **kwargs)


class ResourceBase(adict, metaclass=abc.ABCMeta):
    """Base class for pyrkube API resources."""
    def __init__(self, api, obj, *args, **kwargs):
        if not hasattr(self, 'kind'):
            raise NotImplementedError('%s must have kind attr', type(self))
        wrapper = obj
        obj = copy.deepcopy(wrapper.obj)
        obj['apiVersion'] = obj.get('apiVersion', wrapper.version)

        adict.__init__(self, obj, *args, **kwargs)
        self._wrapper = wrapper
        self._api = api
        self._init_defaults()
        self._deserialize_annotations()

    def _init_defaults(self):
        for key in ('labels', 'annotations'):
            if not self.metadata.get(key):
                self.metadata[key] = adict()

    def _deserialize_annotations(self):
        self.metadata.annotations = adict(
            {key: util.deserialize(val)
             for key, val in self.metadata.annotations.items()}
        )

    @property
    def name(self):
        """Return kubernetes metadata.name value."""
        return self.metadata.get('name')

    def __lt__(self, other):
        return self.name < other.name

    def reload(self):
        """Reload the object's data from the kubernetes API."""
        self = self._api.get(self.kind, self.name)


class Pod(ResourceBase):
    """A Kubernetes Pod Resource."""
    kind = 'Pod'

    def __init__(self, api, obj):
        ResourceBase.__init__(self, api, obj)
        self._init_containers()

    def _init_containers(self):
        self.spec.containers = adict(
            (c.name, Container(self, c)) for c in self.spec.containers)

    @util.cached_property(ttl=30)
    def owner(self):
        """Returns the API object which owns this pod."""
        reference = self.metadata.annotations.get(
            'kubernetes.io/created-by', {}).get('reference')
        if reference:
            return self._api.get(reference.kind, reference.name)

    @property
    def ready(self):
        """Return true if this pod is in a ready state."""
        return (
            all([c.status for c in self.status.conditions]) and
            all([s.ready for s in self.status.containerStatuses]) and
            self.status.phase == 'Running'
        )


class Container(GenericBase):
    """A container within a pod or pod template."""
    def __init__(self, parent, obj):
        GenericBase.__init__(self, obj)
        self._parent = parent
        self._init_environment()

    def _init_environment(self):
        if hasattr(self, 'env'):
            self.env = ContainerEnv(self, {env.name: env for env in self.env})


class ContainerEnv(GenericBase):
    """The environment of a container in a pod or pod template.

    The environment is also resolved for all externally referenced values.
    """

    def __init__(self, container, obj):
        GenericBase.__init__(self, obj)
        self._container = container
        self._api = container._parent._api
        self._init_environment()

    def _init_environment(self):
        for key, val in self.items():
            setattr(self, key, self._lookup(val))

    def _lookup(self, env):
        if env.get('value'):
            return env.value
        elif env.get('valueFrom'):
            if env.valueFrom.get('secretKeyRef'):
                ref = env.valueFrom.secretKeyRef
                return self._api.get('secret', ref.name).get(ref.key)
            elif env.valueFrom.get('configMapKeyRef'):
                ref = env.valueFrom.configMapKeyRef
                return self._api.get('configmap', ref.name).get(ref.key)


class DataContainerBase(ResourceBase):
    """Base class for data container objects such as Secret or ConfigMap."""
    _encoded = None

    def __init__(self, api, obj):
        ResourceBase.__init__(self, api, obj)
        if self._encoded:
            self._decode_data()

    def _decode_data(self):
        for key, value in self.data.items():
            self.data[key] = base64.b64decode(value).decode()

    def get(self, key, default=None):
        """Get value in obj.data."""
        return self.data.get(key, default)


class ConfigMap(DataContainerBase):
    """A kubernetes ConfigMap."""
    kind = 'ConfigMap'
    _encoded = False


class Secret(DataContainerBase):
    """A kubernetes Secret."""
    kind = 'Secret'
    _encoded = True


class ReplicatedBase(ResourceBase):
    """Base Class for a replicated API object type such as Deployment."""
    def __init__(self, api, obj):
        ResourceBase.__init__(self, api, obj)
        self._init_containers()

    def _init_containers(self):
        self.spec.template.spec.containers = adict(
            (cont.name, Container(self, cont))
            for cont in self.spec.template.spec.containers
        )

    @util.cached_property(ttl=10)
    def pods(self):
        """Return the pods targeted by this replicated object template."""
        if self.spec.selector.get('matchLabels'):
            selector = self.spec.selector.get('matchLabels')
            return self._api.get('pod', selector=selector)


class ReplicaSet(ReplicatedBase):
    """A kubernetes ReplicaSet."""
    kind = 'ReplicaSet'

    @property
    def ready(self):
        """Return True if ready."""
        return (
            self.status.observedGeneration >= self.metadata.generation and
            self.status.readyReplicas == self.metadata.annotations.get(
                'deployment.kubernetes.io/desired-replicas'
            ) and
            all([pod.ready for pod in self.pods])
        )


class Deployment(ReplicatedBase):
    """A kubernetes Deployment"""
    kind = 'Deployment'

    @property
    def ready(self):
        """Return True if ready."""
        return (
            self.status.observedGeneration >= self.metadata.generation and
            self.status.updatedReplicas == self.spec.replicas and
            all([pod.ready for pod in self.pods])
        )


class DaemonSet(ReplicatedBase):
    """A kubernetes DaemonSet."""
    kind = 'DaemonSet'


class StatefulSet(ReplicatedBase):
    """A kubernetes StatefulSet."""
    kind = 'StatefulSet'

    @property
    def ready(self):
        """Return True if ready."""
        return (
            self.spec.replicas == self.status.replicas and
            all([pod.ready for pod in self.pods])
        )


class EndpointAddress(GenericBase):
    """A kubernetes Endpoint Address"""
    def __init__(self, endpoint, obj):
        GenericBase.__init__(self, obj)
        self._endpoint = endpoint

    @util.cached_property(ttl=10)
    def target(self):
        """Return the API object targeted by this Endpoint Address."""
        return self._endpoint._api.get(
            self.targetRef.kind, self.targetRef.name
        )


class Endpoint(ResourceBase):
    """A kubernetes Endpoint."""
    kind = 'Endpoints'

    def __init__(self, api, obj):
        ResourceBase.__init__(self, api, obj)
        self.subsets = [
            [EndpointAddress(self, addr) for addr in sub.addresses]
            for sub in self.subsets
        ]

    def _address_attribute(self, key):
        return sorted([getattr(addr, key) for addr in self.addresses])

    @property
    def addresses(self):
        """Return the addresses in this endpoint."""
        if len(self.subsets):
            return self.subsets[0]
        else:
            return self.subsets

    @property
    def ready(self):
        """Return True if all pods targeted by this endpoint are ready."""
        return all([pod.ready for pod in self.pods])

    @property
    def pods(self):
        """Return a list of pod objects targeted by this endpoint."""
        return self._address_attribute('target')

    @property
    def nodes(self):
        """Return a list of nodes targeted by this endpoint."""
        return self._address_attribute('nodeName')

    @property
    def ips(self):
        """Return a list of IPs targeted by this endpoint."""
        return self._address_attribute('ip')

    @property
    def hosts(self):
        """Return a list of hosts targeted by this endpoint."""
        return [host.hostname for host in self.hostnames]

    @property
    def hostnames(self):
        """Return a list of hostname objects targeted by this endpoint."""
        return [hosts.get_hostname(pod) for pod in self.pods]


class Service(ResourceBase):
    """A kubernetes Service."""
    kind = 'Service'

    @property
    def endpoint(self):
        """Return the endpoint object corresponding to this service."""
        return self._api.get('endpoint', self.name)


class KubeApp:
    """Abstraction for a kubernetes app.

    Designed to retrieve an application's readiness state.

    Attributes:
      api: (pyrkube.KubeAPIClient) the api object to use
      name: (str) the name of the deployment or statefulset
      sleep: (int) number of seconds to wait between polling state
    """

    def __init__(self, api, name, sleep=5):
        if not name:
            raise ValueError('name attribute must not be empty')
        self.api = api
        self.name = name
        self.sleep = sleep

    def __repr__(self):
        return '%s(%s %s %s)' % (
            type(self).__name__,
            self.kind,
            self.name,
            ', '.join(('%s=%r' % (a, getattr(self, a))
                       for a in ('loaded', 'ready')))
        )

    @util.cached_property(ttl=600)
    def kind(self):
        """Return the kubernetes api object kind."""
        return (
            self.api.get('deployment', self.name) or
            self.api.get('statefulset', self.name) or {}
        ).get('kind')

    @util.cached_property(ttl=30)
    def resource(self):
        """Return the API resource representing the app."""
        if self.kind:
            return self.api.get(self.kind, self.name)

    @property
    def loaded(self):
        """API Object exists in kubernetes."""
        return bool(self.resource)

    @property
    def ready(self):
        """API Object is in ready state."""
        return bool(self.resource) and self.resource.ready

    def poll(self):
        """Continually poll the state of the application."""
        while True:
            print(self)
            time.sleep(self.sleep)

    def wait(self):
        """Blocking wait until application is ready."""
        while not self.ready:
            time.sleep(self.sleep)

    def __bool__(self):
        return self.ready


class KubeApps(util.WrappedMapMixin):
    """Abstracted container for holding KubeApps.

    Attributes:
      api: (pyrkube.KubeAPIClient) the api object to use
      names: (list,tuple) names of deployment/statefulsets
      logger: (str) the name of a logger object to use.  Set to None for no
              logging
      sleep: (int) number of seconds to wait between polling state
    """
    def __init__(self, api, names, logger='kubeapp', sleep=5):
        if not isinstance(names, (list, tuple)):
            raise TypeError('names provided is not a list or tuple.')
        self.api = api
        self.names = names
        if logger:
            self.logging = True
            self.logger = logging.getLogger(logger)
        else:
            self.logging = False
        self.sleep = sleep
        self._init_apps()

    def _get_app(self, name):
        return KubeApp(self.api, name, self.wait)

    def _init_apps(self):
        self.apps = {a.name: a for a
                     in [self._get_app(n) for n in self.names]}
        self._wrapped = self.apps

    def __repr__(self):
        return '%s(apps: %s)' % (
            type(self).__name__,
            ', '.join(['%s' % app for app in self])
        )

    def __iter__(self):
        return iter(self.values())

    def log(self, *args):
        """If logging is enabled, log the following *args"""
        if self.logging:
            self.logger.info(*args)

    @property
    def status(self):
        """Return a dictionary of application names => ready states."""
        return {app.name: app.ready for app in self}

    @property
    def ready(self):
        """Return True if all apps are ready."""
        return all([app.ready for app in self])

    def poll(self):
        """Continually poll the state of the application."""
        while True:
            print(', '.join(self))
            time.sleep(self.sleep)

    def wait(self):
        """Blocking wait until all apps are ready.

        If logging enabled, detailed logs are output to the logger.
        """
        def log_status():
            """Log's the current status of the wait cycle."""
            msg = []
            if ready:
                msg.append('%d apps ready.' % ready)
            if apps:
                msg.append('waiting for %d apps: %s' % (
                    len(apps), ', '.join([app.name for app in apps])
                ))
            if msg:
                self.log(' '.join(msg))

        self.log('Loaded %d apps: %s', len(self.names), ', '.join(self.names))
        apps = list(self)
        ready = 0
        while apps:
            for app in apps[:]:
                if app.ready:
                    self.log('app: %s ready', app.name)
                    apps.remove(app)
                    ready += 1

            log_status()
            if apps:
                time.sleep(self.sleep)
            else:
                self.log('Finished')



"""
pyrkube.config
~~~~~~~~~~~~~~

Configuration for pyrkube.

:copyright: (c) 2016 by Joe Black.
:license: Apache2.
"""

import os

LOG_LEVEL = os.getenv('LOG_LEVEL', 'DEBUG').upper()
ENVIRONMENT = os.getenv('ENVIRONMENT', 'dev')
DOMAIN = os.getenv('DNS_DOMAIN', 'cluster.local')

del os



import types

import pyrkube

from . import TestCase


class TestClient(TestCase):

    def setUp(self):
        self.env = pyrkube.config.ENVIRONMENT
        self.api = pyrkube.client.KubeAPIClient(
            env=self.env, namespace='default', domain='cluster.local')

    def tearDown(self):
        self.env = None
        self.api = None

    def test_client_setup(self):
        """
        Test Config instance creation.
        """
        api = self.api
        self.assertIn(api.env, ('test', 'dev'))
        self.assertEqual(api.cluster, 'minikube')
        self.assertEqual(api.namespace, 'default')

        self.assertEqual(api.domain, 'cluster.local')
        self.assertEqual(api.user, 'minikube')
        self.assertTrue(api.server.startswith('https://'))

    def test_get_by_name(self):
        pods = self.api.get('pod')
        name = pods[0].name

        pod = self.api.get('pod', name)
        self.assertIsNotNone(pod)
        self.assertIsInstance(pod, pyrkube.objects.Pod)

    def test_get_by_selector(self):
        pods = self.api.get('pod')
        selector = dict(pods[0].metadata.labels)

        pods = self.api.get('pod', selector=selector)
        self.assertIsNotNone(pods)
        self.assertIsInstance(pods, list)
        self.assertGreater(len(pods), 0)

        for pod in pods:
            self.assertIsNotNone(pod)
            self.assertIsInstance(pod, pyrkube.objects.Pod)

    def test_get_watch(self):

        pods = self.api.get('pod')
        selector = dict(pods[0].metadata.labels)

        watch = self.api.get('pod', selector=selector, watch=True)

        self.assertIsNotNone(watch)
        self.assertIsInstance(watch, types.GeneratorType)
